const { ccclass, property } = cc._decorator;

@ccclass
export class GlobalTouchListener extends cc.Component {

    private left: number = 0;
    private right: number = 0;
    private top: number = 0;
    private bottom: number = 0;

    private touchStartCallbackMap: Map<Function, any> = new Map<Function, any>();
    private touchMoveCallbackMap: Map<Function, any> = new Map<Function, any>();
    private touchEndCallbackMap: Map<Function, any> = new Map<Function, any>();
    private touchCancelCallbackMap: Map<Function, any> = new Map<Function, any>();

    onLoad() {
        let newTouchListener = cc["EventListener"].create({
            event: cc["EventListener"].TOUCH_ONE_BY_ONE,
            swallowTouches: false,
            owner: this,
            onTouchBegan: (touch: cc.Touch, event: cc.Event) => {
                if (this.isTouchInArea(touch)) {
                    this.touchStartCallbackMap.forEach((target: any, callback: Function) => {
                        callback.call(target, touch, event);
                    });
                    return true;
                }
                else return false;
            },
            onTouchMoved: (touch: cc.Touch, event: cc.Event) => {
                this.touchMoveCallbackMap.forEach((target: any, callback: Function) => {
                    callback.call(target, touch, event);
                });
            },
            onTouchEnded: (touch: cc.Touch, event: cc.Event) => {
                if (this.isTouchInArea(touch)) {
                    this.touchEndCallbackMap.forEach((target: any, callback: Function) => {
                        callback.call(target, touch, event);
                    });
                }
                else {
                    this.touchCancelCallbackMap.forEach((target: any, callback: Function) => {
                        callback.call(target, touch, event);
                    });
                }
            }
        });
        cc["internal"].eventManager.addListener(newTouchListener, -1);
    }

    start() {
        this.left = this.node.x - this.node.width / 2;
        this.right = this.node.x + this.node.width / 2;
        this.top = this.node.y + this.node.height / 2;
        this.bottom = this.node.y - this.node.height / 2;
    }

    public on(type: cc.Node.EventType, call: Function, target?: any): void {
        switch (type) {
            case cc.Node.EventType.TOUCH_START:
                this.touchStartCallbackMap.set(call, target);
                break;
            case cc.Node.EventType.TOUCH_MOVE:
                this.touchMoveCallbackMap.set(call, target);
                break;
            case cc.Node.EventType.TOUCH_END:
                this.touchEndCallbackMap.set(call, target);
                break;
            case cc.Node.EventType.TOUCH_CANCEL:
                this.touchCancelCallbackMap.set(call, target);
                break;
            default:
                break;
        }
    }

    public off(type: cc.Node.EventType, callback?: Function, target?: any): void {
        switch (type) {
            case cc.Node.EventType.TOUCH_START:
                if (callback == null) this.touchStartCallbackMap.clear();
                else if (this.touchStartCallbackMap.has(callback) && (target == null || target === this.touchStartCallbackMap.get(callback))) this.touchStartCallbackMap.delete(callback);
                break;
            case cc.Node.EventType.TOUCH_MOVE:
                if (callback == null) this.touchMoveCallbackMap.clear();
                else if (this.touchMoveCallbackMap.has(callback) && (target == null || target === this.touchMoveCallbackMap.get(callback))) this.touchMoveCallbackMap.delete(callback);
                break;
            case cc.Node.EventType.TOUCH_END:
                if (callback == null) this.touchEndCallbackMap.clear();
                else if (this.touchEndCallbackMap.has(callback) && (target == null || target === this.touchEndCallbackMap.get(callback))) this.touchEndCallbackMap.delete(callback);
                break;
            case cc.Node.EventType.TOUCH_CANCEL:
                if (callback == null) this.touchCancelCallbackMap.clear();
                else if (this.touchCancelCallbackMap.has(callback) && (target == null || target === this.touchCancelCallbackMap.get(callback))) this.touchCancelCallbackMap.delete(callback);
                break;
            default:
                break;
        }
    }

    private isTouchInArea(touch: cc.Touch): boolean {
        let worldPos = touch.getLocation();
        let nodePos = this.node.parent.convertToNodeSpaceAR(worldPos);

        let x = nodePos.x;
        let y = nodePos.y;

        if (x < this.left || x > this.right || y > this.top || y < this.bottom) return false;
        else return true;
    }
}